{*****************************************************************************
*                                                                            *
*  CUBE1.DPR                                                                 *
*  CUBE1U.PAS                                                                *
*                                                                            *
*  This program uses Fastgraph's 3D geometry system to draw a cube. Each     *
*  of the six cube faces is a different solid color. Uses the approach of    *
*  projecting each cube face separately.                                     *
*                                                                            *
*****************************************************************************}

unit Cube1U;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, FGWin;

type
  TForm1 = class(TForm)
    procedure AppOnActivate(Sender: TObject);
    procedure AppIdle(Sender: TObject; var Done: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
type
  FIXED = longint;
  POINT2D = record
    x : integer;
    y : integer;
  end;
  POINT3D = record
    x : FIXED;
    y : FIXED;
    z : FIXED;
  end;

const
  { virtual buffer dimensions }
  vbWidth  : integer = 640;
  vbHeight : integer = 480;

  { six faces of a 40x40x40 cube, defined in 3D coordinates }
  face1 : array [1..4] of POINT3D = (
    (x:$FFEC0000; y:$00140000; z:$00140000),
    (x:$00140000; y:$00140000; z:$00140000),
    (x:$00140000; y:$FFEC0000; z:$00140000),
    (x:$FFEC0000; y:$FFEC0000; z:$00140000));
  face2 : array [1..4] of POINT3D = (
    (x:$00140000; y:$00140000; z:$00140000),
    (x:$00140000; y:$00140000; z:$FFEC0000),
    (x:$00140000; y:$FFEC0000; z:$FFEC0000),
    (x:$00140000; y:$FFEC0000; z:$00140000));
  face3 : array [1..4] of POINT3D = (
    (x:$FFEC0000; y:$FFEC0000; z:$FFEC0000),
    (x:$00140000; y:$FFEC0000; z:$FFEC0000),
    (x:$00140000; y:$00140000; z:$FFEC0000),
    (x:$FFEC0000; y:$00140000; z:$FFEC0000));
  face4 : array [1..4] of POINT3D = (
    (x:$FFEC0000; y:$00140000; z:$FFEC0000),
    (x:$FFEC0000; y:$00140000; z:$00140000),
    (x:$FFEC0000; y:$FFEC0000; z:$00140000),
    (x:$FFEC0000; y:$FFEC0000; z:$FFEC0000));
  face5 : array [1..4] of POINT3D = (
    (x:$FFEC0000; y:$00140000; z:$FFEC0000),
    (x:$00140000; y:$00140000; z:$FFEC0000),
    (x:$00140000; y:$00140000; z:$00140000),
    (x:$FFEC0000; y:$00140000; z:$00140000));
  face6 : array [1..4] of POINT3D = (
    (x:$FFEC0000; y:$FFEC0000; z:$00140000),
    (x:$00140000; y:$FFEC0000; z:$00140000),
    (x:$00140000; y:$FFEC0000; z:$FFEC0000),
    (x:$FFEC0000; y:$FFEC0000; z:$FFEC0000));

  { for convenience, an array of pointers to each of the six faces }
  faces : array [1..6] of ^POINT3D = (@face1,@face2,@face3,@face4,@face5,@face6);

var
  { 3D transformation matrix }
  transform : array [1..12] of FIXED;

  { work array required by fg_polyfill() }
  work_array : array [1..1000] of POINT2D;

  cxClient, cyClient : integer;
  dc   : hDC;
  hpal : hPalette;
  hvb  : integer;
  bpp  : integer;
  redo : boolean;
  xAngle, yAngle, zAngle : integer;

{*****************************************************************************
*                                                                            *
*  draw_cube()                                                               *
*                                                                            *
*  Projects and draws each of the six cube faces.                            *
*                                                                            *
*****************************************************************************}

procedure draw_cube;

const
  colors : array [1..6] of integer = (84,88,92,96,100,104);

var
  i : integer;
  r, g, b : integer;
  projected_face : array [1..4] of POINT2D;

begin
  for i := 1 to 6 do
  begin
    fg_project(transform,faces[i]^,projected_face,4);
    if (bpp > 8) then
    begin
      fg_getrgb(colors[i],r,g,b);
      fg_setcolor(fg_maprgb(r,g,b));
    end
    else
      fg_setcolor(colors[i]);
    fg_polyfill(projected_face,work_array,-4);
  end;
end;

{*****************************************************************************
*                                                                            *
*  check_for_movement()                                                      *
*                                                                            *
*  The check_for_movement() function checks if any of the four arrow keys    *
*  are pressed, and if so, pans in that direction if possible. It is called  *
*  from the application's OnIdle event handler.                              *
*                                                                            *
*****************************************************************************}

procedure check_for_movement;

var
  shift_key : boolean;

begin
  { erase the previous frame from the virtual buffer }
  fg_setcolor(-1);
  fg_fillpage;

  { project the cube onto the virtual buffer and draw it }
  draw_cube;

  { display what we just drew }
  fg_vbscale(0,vbWidth-1,0,vbHeight-1,0,cxClient-1,0,cyClient-1);

  { check if either shift key is pressed }
  shift_key := (fg_kbtest(42) = 1) or (fg_kbtest(54) = 1);

  { + and - move cube along the z axis (+ is toward viewer, - is }
  { away from viewer), restricting movement beyond z=-40 }
  if (fg_kbtest(74) = 1) then
    fg_move3d(transform,0,0,$FFFD0000,1)
  else if (fg_kbtest(78) = 1) then
  begin
    if (transform[12] < $FFD80000) then
      fg_move3d(transform,0,0,$00030000,1);
  end

  { left and right arrow keys move cube along x axis }
  else if (fg_kbtest(75) = 1) then
    fg_move3d(transform,$FFFD0000,0,0,1)
  else if (fg_kbtest(77) = 1) then
    fg_move3d(transform,$00030000,0,0,1)

  { up and down arrow keys move cube along y axis }
  else if (fg_kbtest(72) = 1) then
    fg_move3d(transform,0,$00030000,0,1)
  else if (fg_kbtest(80) = 1) then
    fg_move3d(transform,0,$FFFD0000,0,1)

  { x rotates counterclockwise around x axis, X rotates clockwise }
  else if (fg_kbtest(45) = 1) then
  begin
    if (shift_key) then
    begin
      inc(xAngle,6);
      if (xAngle >= 360) then dec(xAngle,360);
    end
    else
    begin
      dec(xAngle,6);
      if (xAngle < 0) then inc(xAngle,360);
    end;
    redo := True;
  end

  { y rotates counterclockwise around y axis, Y rotates clockwise }
  else if (fg_kbtest(21) = 1) then
  begin
    if (shift_key) then
    begin
      inc(yAngle,6);
      if (yAngle >= 360) then dec(yAngle,360);
    end
    else
    begin
      dec(yAngle,6);
      if (yAngle < 0) then inc(yAngle,360);
    end;
    redo := True;
  end

  { z rotates counterclockwise around z axis, Z rotates clockwise }
  else if (fg_kbtest(44) = 1) then
  begin
    if (shift_key) then
    begin
      inc(zAngle,6);
      if (zAngle >= 360) then dec(zAngle,360);
    end
    else
    begin
      dec(zAngle,6);
      if (zAngle < 0) then inc(zAngle,360);
    end;
    redo := True;
  end;

  { if rotation changed, modify the transformation matrix accordingly }
  if (redo) then
  begin
    fg_rotate3d(transform,xAngle*10,yAngle*10,zAngle*10,0);
    redo := False;
  end;

end;

{****************************************************************************}

procedure TForm1.AppOnActivate(Sender: TObject);
begin
  fg_realize(hpal);
  Invalidate;
end;

procedure TForm1.AppIdle(Sender: Tobject; var Done: Boolean);
begin
  check_for_movement;
  Done := False;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  fg_realize(hpal);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  dc := GetDC(Form1.Handle);
  fg_setdc(dc);
  hpal := fg_defpal;
  fg_realize(hpal);

  fg_vbinit;
{$IFDEF VER80}
  bpp := 8;
{$ELSE}
  bpp := fg_colors;
  if (bpp > 8) then fg_vbdepth(bpp);
{$ENDIF}
  hvb := fg_vballoc(vbWidth,vbHeight);
  fg_vbopen(hvb);
  fg_vbcolors;
  fg_setcolor(-1);
  fg_fillpage;

  { define 3D viewport and initial transformation matrix }
  fg_view3d(0,vbWidth-1,0,vbHeight-1,$00008000);
  fg_rotate3d(transform,0,0,0,0);
  fg_move3d(transform,0,0,$FF9C0000,0);

  redo := False;
  xAngle := 0;
  yAngle := 0;
  zAngle := 0;

  Application.OnActivate := AppOnActivate;
  Application.OnIdle := AppIdle;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  fg_vbscale(0,vbWidth-1,0,vbHeight-1,0,cxClient-1,0,cyClient-1);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  cxClient := ClientWidth;
  cyClient := ClientHeight;
  Invalidate;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  fg_vbclose;
  fg_vbfree(hvb);
  fg_vbfin;
  DeleteObject(hpal);
  ReleaseDC(Form1.Handle,dc);
end;

end.
