!*****************************************************************************
!                                                                            *
!  FISHTANK.F90                                                              *
!                                                                            *
!  This program shows how to perform simple animation using Fastgraph for    *
!  Windows. Several types of tropical fish swim back and forth against a     *
!  coral reef background. The background image and fish sprites are stored   *
!  in standard pixel run (SPR) files.                                        *
!                                                                            *
!  The application window is initially set to 320x200 pixels, the same size  *
!  as the CORAL.SPR image. When the window size changes, the image is scaled *
!  to fill the window.                                                       *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGfishtank'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW .or. CS_OWNDC
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = NULL
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Fastgraph Fish Tank'C,  &  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   320,                     &  ! initial x size
   200,                     &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

! The message loop processes entries placed in the message queue.
! When no message is ready, call scroll() to perform one frame of
! animation.

do while (.TRUE.)
   if (PeekMessage(msg,NULL,0,0,PM_REMOVE)) then
      if (msg.message == WM_QUIT) then
         WinMain = msg%wParam
         return
      else
         iret = TranslateMessage(msg)
         iret = DispatchMessage(msg)
      end if
   else
      call go_fish
   end if
end do

end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'fishtank.fi'

type (T_PAINTSTRUCT) ps
integer*4 hdc, hpal
integer*4 iret

save hdc, hpal

WindowProc = 0

select case (message)

   case (WM_CREATE)
      ! use the default logical palette
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      ! create two 320x200 virtual buffers
      call fg_vbinit
      hvb1 = fg_vballoc(320,200)
      hvb2 = fg_vballoc(320,200)

      ! display the coral background in virtual buffer #2 (which
      ! will always contain a clean copy of the background image)
      iret = fg_vbopen(hvb2)
      call fg_vbcolors
      call fg_move(0,199)
      iret = fg_showspr('CORAL.SPR'C,320)

      ! get the fish bitmaps
      call get_fish
      return

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb1)
      call fg_vbfree(hvb2)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return

end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************
!                                                                            *
!  get_fish                                                                  *
!                                                                            *
!  Fill the fish bitmap arrays.                                              *
!                                                                            *
!*****************************************************************************

subroutine get_fish
use fgwin

include 'fishtank.fi'

integer*4 i, j

! get the fish bitmaps from an SPR file
i = fg_vbopen(hvb1)
call fg_vbcolors
call fg_move(0,199)
i = fg_showspr('FISH.SPR'C,320)

j = 1
do i = 1,6
   call fg_move(fish_x(i),fish_y(i))
   call fg_getimage(fishes(j),width(i),height(i))
   offset(i) = j
   j = j + width(i) * height(i)
end do

call fg_erase

return
end

!*****************************************************************************
!                                                                            *
!  go_fish                                                                   *
!                                                                            *
!  Make the fish swim around.                                                *
!                                                                            *
!*****************************************************************************

subroutine go_fish
use fgwin

include 'fishtank.fi'

integer*4 i

! copy the background to the workspace
call fg_copypage(hvb2,hvb1)

! put all the fish in their new positions
do i = 1,NFISH
   ycount(i) = ycount(i) + 1
   if (ycount(i) > yturn(i)) then
      ycount(i) = 0
      yinc(i) = MOD(IRAND(),3) - 1
   end if
   y(i) = y(i) + yinc(i)
   y(i) = MIN(ymax(i),MAX(y(i),ymin(i)))

   if (x(i) >= -72 .and. x(i) < 320) then
      call put_fish(fish(i),x(i),y(i),dir(i))
   end if

   x(i) = x(i) + xinc(i)
   if (x(i) <= xmin(i) .or. x(i) >= xmax(i)) then
      xinc(i) = -xinc(i)
      dir(i) = 1 - dir(i)
   end if
end do

! scale the workspace image to fill the client area
call fg_vbscale(0,319,0,199,0,cxClient-1,0,cyClient-1)

return
end

!*****************************************************************************
!                                                                            *
!  put_fish                                                                  *
!                                                                            *
!  Draw one of the six fish anywhere you want.                               *
!                                                                            *
!*****************************************************************************

subroutine put_fish (fish_num, xpos, ypos, fish_dir)
use fgwin
integer*4 fish_num, xpos, ypos, fish_dir

include 'fishtank.fi'

integer*4 i

! move to position where the fish will appear
call fg_move(xpos,ypos)

! draw a left- or right-facing fish, depending on fish_dir
i = offset(fish_num)
if (fish_dir == 0) then
   call fg_flpimage(fishes(i),width(fish_num),height(fish_num))
else
   call fg_clpimage(fishes(i),width(fish_num),height(fish_num))
end if

return
end

!*****************************************************************************

block data

include 'fishtank.fi'

data fish_x /  0, 64,128,200,  0, 80/  ! location of fish (x)
data fish_y /199,199,199,199,150,150/  ! location of fish (y)
data width  / 56, 54, 68, 56, 62, 68/  ! size of fish: width
data height / 25, 38, 26, 30, 22, 36/  ! size of fish: height

! which fish bitmap applies to this fish?
data fish  /   2,   2,   3,   4,   4,   1,   1,   6,   5,   3,   4/
! starting x coordinate
data x     /-100,-150,-450,-140,-200, 520, 620,-800, 800, 800,-300/
! starting y coordinate
data y     /  40,  60, 150,  80,  70, 190, 180, 100,  30, 130,  92/

! how far left (off screen) the fish can go
data xmin  /-300,-300,-800,-200,-200,-200,-300,-900,-900,-900,-400/
! how far right (off screen) the fish can go
data xmax  / 600, 600,1100,1000,1000, 750, 800,1200,1400,1200, 900/
! how fast the fish goes left and right
data xinc  /   2,   2,   8,   5,   5,  -3,  -3,   7,  -8,  -9,   6/
! starting direction for each fish
data dir   /   0,   0,   0,   0,   0,   1,   1,   0,   1,   1,   0/

! how far up this fish can go
data ymin  /  40,  60, 120,  70,  60, 160, 160,  80,  30, 110,  72/
! how far down this fish can go
data ymax  /  80, 100, 170, 110, 100, 199, 199, 120,  70, 150, 122/
! how long fish can go vertically before stopping or turning around
data yturn /  50,  30,  10,  30,  20,  10,  10,  10,  30,   20, 10/
! counter to compare to yturn
data ycount/   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0/
! how fast the fish moves up or down
data yinc  /   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0/

end
