/****************************************************************************\
*                                                                            *
*  FRAMEDD.C                                                                 *
*                                                                            *
*  This program shows how to set up a full screen DirectDraw application     *
*  for either blitting or flipping. The selection of blitting or flipping is *
*  controlled by the BLIT and FLIP symbols defined below.                    *
*                                                                            *
\****************************************************************************/

#include <fgwin.h>

#define WIDTH  640
#define HEIGHT 480

// define either BLIT or FLIP, but not both, for blitting or flipping
#define BLIT
//#define FLIP

BOOL AppIsActive = FALSE;

long WINAPI _export WindowProc(HWND,UINT,UINT,LONG);
void animate(void);

int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR lpszCmdParam, int nCmdShow)
{
   static char szAppName[] = "FGframedd";
   HWND        window;
   MSG         msg;
   WNDCLASS    wndclass;

   if (!hPrevInstance)
   {
      wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
      wndclass.lpfnWndProc   = WindowProc;
      wndclass.cbClsExtra    = 0;
      wndclass.cbWndExtra    = 0;
      wndclass.hInstance     = hInstance;
      wndclass.hIcon         = LoadIcon(NULL,IDI_APPLICATION);
      wndclass.hCursor       = LoadCursor(NULL,IDC_ARROW);
      wndclass.hbrBackground = NULL;
      wndclass.lpszMenuName  = NULL;
      wndclass.lpszClassName = szAppName;
      RegisterClass (&wndclass);
   }

   window = CreateWindowEx(
      WS_EX_TOPMOST,                // extended window style
      szAppName,                    // window class name
      "DirectDraw Frame Animation", // window caption
      WS_POPUP,                     // window style
      0,                            // initial x position
      0,                            // initial y position
      WIDTH,                        // initial x size
      HEIGHT,                       // initial y size
      NULL,                         // parent window handle
      NULL,                         // window menu handle
      hInstance,                    // program instance handle
      NULL);                        // creation parameters

   ShowWindow(window,nCmdShow);
   UpdateWindow(window);

   // The message loop processes entries placed in the message queue.
   // When no message is ready and this is the active application, call
   // animate() to perform one frame of animation.

   while (TRUE)
   {
      if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
      {
         if (msg.message == WM_QUIT)
            break;
         else
         {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
         }
      }
      else if (AppIsActive)
         animate();
   }

   return msg.wParam;
}

/****************************************************************************\
*                                                                            *
*  WindowProc                                                                *
*                                                                            *
*  Window procedure to handle messages sent to the window.                   *
*                                                                            *
\****************************************************************************/

HDC      hdc;
HPALETTE hpal;
int      hvb;

long WINAPI _export WindowProc(HWND window, UINT message,
                               UINT wParam, LONG lParam)
{
   switch (message)
   {
      case WM_ACTIVATEAPP:
         AppIsActive = wParam;
         return 0;

      case WM_CREATE:
#ifdef BLIT
         fg_ddsetup(WIDTH,HEIGHT,8,0);
#else
         fg_ddsetup(WIDTH,HEIGHT,8,1);
#endif
         hdc = GetDC(window);
         fg_setdc(hdc);
         hpal = fg_defpal();
         fg_realize(hpal);
         ShowWindow(window,SW_SHOWNORMAL);

         // if blitting, create a virtual buffer the same size as the screen
         // resolution; if flipping, use the primary surface's back buffer
         fg_vbinit();
#ifdef BLIT
         hvb = fg_vballoc(WIDTH,HEIGHT);
#else
         hvb = 0;
#endif
         fg_vbopen(hvb);
         fg_vbcolors();

         fg_mouseini();
         fg_mousevis(0);
         AppIsActive = TRUE;
         return 0;

      case WM_KEYDOWN:
         switch(wParam)
         {
            case VK_ESCAPE:
            case VK_F12:
               DestroyWindow(window);
               break;
         }
         return 0;

      case WM_SETFOCUS:
         fg_realize(hpal);
         InvalidateRect(window,NULL,TRUE);
         return 0;

      case WM_DESTROY:
         fg_mousevis(1);
         fg_vbclose();
#ifdef BLIT
         fg_vbfree(hvb);
#endif
         fg_vbfin();
         DeleteObject(hpal);
         ReleaseDC(window,hdc);
         PostQuitMessage(0);
         return 0;
   }
   return DefWindowProc(window,message,wParam,lParam);
}

/****************************************************************************\
*                                                                            *
*  animate                                                                   *
*                                                                            *
*  Construct the next frame of animation and display it with either blitting *
*  or flipping, as directed by the BLIT and FLIP symbols above.              *
*                                                                            *
\****************************************************************************/

void animate(void)
{
   // fill drawing surface with the next color
   fg_setcolor((fg_getcolor() + 1) & 0xFF);
   fg_fillpage();

   // blit or flip surface to the screen
#ifdef BLIT
   fg_vbpaste(0,WIDTH-1,0,HEIGHT-1,0,HEIGHT-1);
#else
   fg_ddflip();
#endif
}
