!*****************************************************************************
!                                                                            *
!  FRAMEDD.F90                                                               *
!                                                                            *
!  This program shows how to set up a full screen DirectDraw application     *
!  for either blitting or flipping. The selection of blitting or flipping is *
!  controlled by the BLIT and FLIP symbols defined below.                    *
!                                                                            *
!*****************************************************************************

! define either BLIT or FLIP, but not both, for blitting or flipping
$DEFINE BLIT
!$DEFINE FLIP

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

include 'framedd.fi'

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

logical AppIsActive
common  AppIsActive

data AppName /'FGframedd'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW .or. CS_OWNDC
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = NULL
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindowEx(    &
   WS_EX_TOPMOST,           &  ! extended window style
   AppName,                 &  ! window class name
   'DirectDraw Frame Animation'C, & ! window caption
   WS_POPUP,                &  ! window style
   0,                       &  ! initial x position
   0,                       &  ! initial y position
   WIDTH,                   &  ! initial x size
   HEIGHT,                  &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

! The message loop processes entries placed in the message queue.
! When no message is ready and this is the active application, call
! animate() to perform one frame of animation.

do while (.TRUE.)
   if (PeekMessage(msg,NULL,0,0,PM_REMOVE)) then
      if (msg.message == WM_QUIT) then
         WinMain = msg%wParam
         return
      else
         iret = TranslateMessage(msg)
         iret = DispatchMessage(msg)
      end if
   else if (AppIsActive) then
      call animate
   end if
end do

return
end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'framedd.fi'

type (T_PAINTSTRUCT) ps
integer*4 hdc, hpal, hvb
integer*4 iret

logical AppIsActive
common  AppIsActive

save hdc, hpal, hvb

WindowProc = 0

select case (message)

   case (WM_ACTIVATEAPP)
      AppIsActive = wParam
      return

   case (WM_CREATE)
$IF DEFINED (BLIT)
      call fg_ddsetup(WIDTH,HEIGHT,8,0)
$ELSE
      call fg_ddsetup(WIDTH,HEIGHT,8,1)
$ENDIF
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)
      iret = ShowWindow(window,SW_SHOWNORMAL)

      ! if blitting, create a virtual buffer the same size as the screen
      ! resolution; if flipping, use the primary surface's back buffer
      call fg_vbinit
$IF DEFINED (BLIT)
      hvb = fg_vballoc(WIDTH,HEIGHT)
$ELSE
      hvb = 0
$ENDIF
      iret = fg_vbopen(hvb)
      call fg_vbcolors()

      iret = fg_mouseini()
      call fg_mousevis(0)
      AppIsActive = .TRUE.
      return

   case (WM_KEYDOWN)
      if (wParam == VK_ESCAPE .or. wParam == VK_F12) then
         iret = DestroyWindow(window)
      end if
      return

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbpaste(0,WIDTH-1,0,HEIGHT-1,0,HEIGHT-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_DESTROY)
      call fg_mousevis(1)
      call fg_vbclose
$IF DEFINED (BLIT)
      call fg_vbfree(hvb)
$ENDIF
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return

end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************
!                                                                            *
!  animate                                                                   *
!                                                                            *
!  Construct the next frame of animation and display it with either blitting *
!  or flipping, as directed by the BLIT and FLIP symbols above.              *
!                                                                            *
!*****************************************************************************

subroutine animate
use fgwin

include 'framedd.fi'

$IF DEFINED (FLIP)
integer*4 iret
$ENDIF

! fill drawing surface with the next color
call fg_setcolor((fg_getcolor() + 1) .and. #FF)
call fg_fillpage

! blit or flip surface to the screen
$IF DEFINED (BLIT)
call fg_vbpaste(0,WIDTH-1,0,HEIGHT-1,0,HEIGHT-1)
$ELSE
iret = fg_ddflip()
$ENDIF

return
end
