/****************************************************************************\
*                                                                            *
*  IMAGE.CPP                                                                 *
*  IMAGEU.CPP                                                                *
*                                                                            *
*  This program demonstrates the Fastgraph for Windows image file display    *
*  and creation functions.                                                   *
*                                                                            *
\****************************************************************************/

#include <vcl\vcl.h>
#pragma hdrstop

#include "ImageU.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormActivate(TObject *Sender)
{
   fg_realize(hpal);
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
   hdc = GetDC(Form1->Handle);
   fg_setdc(hdc);
   hpal = fg_defpal();
   fg_realize(hpal);

   fg_vbinit();
   hvb = fg_vballoc(4,4);
   fg_vbopen(hvb);
   fg_vbcolors();

   fg_setcolor(25);
   fg_fillpage();

   Application->OnActivate = OnActivate;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormPaint(TObject *Sender)
{
   fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormResize(TObject *Sender)
{
   cxClient = ClientWidth;
   cyClient = ClientHeight;
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
   fg_vbclose();
   fg_vbfree(hvb);
   fg_vbfin();
   DeleteObject(hpal);
   ReleaseDC(Form1->Handle,hdc);
}

/****************************************************************************\
*                                                                            *
*  switch_buffers                                                            *
*                                                                            *
*  Close the and release the active virtual buffer, then create and open a   *
*  new virtual buffer to hold the new image file.                            *
*                                                                            *
\****************************************************************************/

void __fastcall TForm1::switch_buffers(void)
{
   fg_vbclose();
   fg_vbfree(hvb);
   if (colors == 0)
      fg_vbdepth(24);
   else
      fg_vbdepth(8);
   hvb = fg_vballoc(cxBuffer,cyBuffer);
   fg_vbopen(hvb);
   fg_vbcolors();
}

/****************************************************************************\
*                                                                            *
*  Event handlers for the items on the BMP menu.                             *
*                                                                            *
\****************************************************************************/

void __fastcall TForm1::OpenBMPClick(TObject *Sender)
{
   OpenDialog1->DefaultExt = "bmp";
   OpenDialog1->FileName = "";
   OpenDialog1->Filter = "BMP files (*.BMP)|*.BMP";
   OpenDialog1->Options << ofReadOnly;
   if (OpenDialog1->Execute())
   {
      strcpy(file_name,OpenDialog1->FileName.c_str());
      if (fg_bmphead(file_name,file_header) < 0)
      {
         wsprintf(mb_text,"%s is not a BMP file.",(LPSTR)file_name);
         Application->MessageBox(mb_text,"Error",MB_ICONSTOP|MB_OK);
         return;
      }
      Form1->Cursor = crHourGlass;
      colors = fg_bmppal(file_name,file_palette);
      fg_bmpsize(file_header,&cxBuffer,&cyBuffer);
      switch_buffers();
      fg_showbmp(file_name,0);
      fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
      Form1->Cursor = crDefault;

      MakeBMP->Enabled = True;
      InfoBMP->Enabled = True;
      MakePCX->Enabled = True;
      InfoPCX->Enabled = False;
      PlayFLIC->Enabled = False;
      FrameFLIC->Enabled = False;
      ResetFLIC->Enabled = False;
      InfoFLIC->Enabled = False;
      InfoJPEG->Enabled = False;
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::MakeBMPClick(TObject *Sender)
{
   int color_depth;

   SaveDialog1->DefaultExt = "bmp";
   SaveDialog1->FileName = file_name;
   strcpy(strrchr(SaveDialog1->FileName.c_str(),'.'),".bmp");
   SaveDialog1->Filter = "BMP files (*.BMP)|*.BMP";
   SaveDialog1->Options << ofHideReadOnly;
   SaveDialog1->Options << ofOverwritePrompt;
   SaveDialog1->Options << ofPathMustExist;
   if (SaveDialog1->Execute())
   {
      strcpy(file_name,SaveDialog1->FileName.c_str());
      if (colors == 0)
         color_depth = 24;
      else if (colors == 256)
         color_depth = 8;
      else if (colors == 16)
         color_depth = 4;
      else
         color_depth = 1;
      Form1->Cursor = crHourGlass;
      fg_makebmp(0,cxBuffer-1,0,cyBuffer-1,color_depth,file_name);
      Form1->Cursor = crDefault;
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::InfoBMPClick(TObject *Sender)
{
   MBText = "";
   MBText = MBText + file_name + "\n" + cxBuffer + "x" + cyBuffer + " pixels\n";
   if (colors > 0)
      MBText = MBText + colors + " colors";
   else
      MBText = MBText + "24-bit RGB";
   ShowMessage(MBText);
}

/****************************************************************************\
*                                                                            *
*  Event handlers for the items on the PCX menu.                             *
*                                                                            *
\****************************************************************************/

void __fastcall TForm1::OpenPCXClick(TObject *Sender)
{
   OpenDialog1->DefaultExt = "pcx";
   OpenDialog1->FileName = "";
   OpenDialog1->Filter = "PCX files (*.PCX)|*.PCX";
   OpenDialog1->Options << ofReadOnly;
   if (OpenDialog1->Execute())
   {
      strcpy(file_name,OpenDialog1->FileName.c_str());
      if (fg_pcxhead(file_name,file_header) < 0)
      {
         wsprintf(mb_text,"%s is not a PCX file.",(LPSTR)file_name);
         Application->MessageBox(mb_text,"Error",MB_ICONSTOP|MB_OK);
         return;
      }
      Form1->Cursor = crHourGlass;
      colors = fg_pcxpal(file_name,file_palette);
      fg_pcxsize(file_header,&cxBuffer,&cyBuffer);
      switch_buffers();
      fg_move(0,0);
      fg_showpcx(file_name,2);
      fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
      Form1->Cursor = crDefault;

      MakeBMP->Enabled = True;
      InfoBMP->Enabled = False;
      MakePCX->Enabled = True;
      InfoPCX->Enabled = True;
      PlayFLIC->Enabled = False;
      FrameFLIC->Enabled = False;
      ResetFLIC->Enabled = False;
      InfoFLIC->Enabled = False;
      InfoJPEG->Enabled = False;
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::MakePCXClick(TObject *Sender)
{
   SaveDialog1->DefaultExt = "pcx";
   SaveDialog1->FileName = file_name;
   strcpy(strrchr(SaveDialog1->FileName.c_str(),'.'),".pcx");
   SaveDialog1->Filter = "PCX files (*.PCX)|*.PCX";
   SaveDialog1->Options << ofHideReadOnly;
   SaveDialog1->Options << ofOverwritePrompt;
   SaveDialog1->Options << ofPathMustExist;
   if (SaveDialog1->Execute())
   {
      strcpy(file_name,SaveDialog1->FileName.c_str());
      Form1->Cursor = crHourGlass;
      fg_makepcx(0,cxBuffer-1,0,cyBuffer-1,file_name);
      Form1->Cursor = crDefault;
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::InfoPCXClick(TObject *Sender)
{
   MBText = "";
   MBText = MBText + file_name + "\n" + cxBuffer + "x" + cyBuffer + " pixels\n";
   if (colors > 0)
      MBText = MBText + colors + " colors";
   else
      MBText = MBText + "24-bit RGB";
   ShowMessage(MBText);
}

/****************************************************************************\
*                                                                            *
*  Event handlers for the items on the FLI/FLC menu.                         *
*                                                                            *
\****************************************************************************/

void __fastcall TForm1::OpenFLICClick(TObject *Sender)
{
   OpenDialog1->DefaultExt = "fli";
   OpenDialog1->FileName = "";
   OpenDialog1->Filter = "flic files (*.FLI,*.FLC)|*.FLI;*.FLC";
   OpenDialog1->Options << ofReadOnly;
   if (OpenDialog1->Execute())
   {
      strcpy(file_name,OpenDialog1->FileName.c_str());
      if (fg_flichead(file_name,file_header) < 0)
      {
         wsprintf(mb_text,"%s is not an FLI or FLC file.",(LPSTR)file_name);
         Application->MessageBox(mb_text,"Error",MB_ICONSTOP|MB_OK);
         return;
      }
      colors = 256;
      fg_flicsize(file_header,&cxBuffer,&cyBuffer);
      switch_buffers();
      fg_flicopen(file_name,context);
      fg_flicplay(context,1,0);
      fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
      memcpy(&frames,&file_header[6],2);

      MakeBMP->Enabled = True;
      InfoBMP->Enabled = False;
      MakePCX->Enabled = True;
      InfoPCX->Enabled = False;
      PlayFLIC->Enabled = True;
      FrameFLIC->Enabled = True;
      ResetFLIC->Enabled = True;
      InfoFLIC->Enabled = True;
      InfoJPEG->Enabled = False;
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::PlayFLICClick(TObject *Sender)
{
   Form1->Cursor = crHourGlass;
   fg_showflic(file_name,0,1);
   Form1->Cursor = crDefault;
   fg_flicskip(context,-1);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FrameFLICClick(TObject *Sender)
{
   if (fg_flicplay(context,1,0) == 0)
   {
      fg_flicskip(context,-1);
      fg_flicplay(context,1,0);
   }
   fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ResetFLICClick(TObject *Sender)
{
   fg_flicskip(context,-1);
   fg_flicplay(context,1,0);
   fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::InfoFLICClick(TObject *Sender)
{
   MBText = "";
   MBText = MBText
            + file_name + "\n"
            + cxBuffer + "x" + cyBuffer + " pixels\n"
            + frames + " frames";
   ShowMessage(MBText);
}

/****************************************************************************\
*                                                                            *
*  Event handlers for the items on the JPEG menu.                            *
*                                                                            *
\****************************************************************************/

void __fastcall TForm1::OpenJPEGClick(TObject *Sender)
{
   char *buffer;
   int buffsize;

   OpenDialog1->DefaultExt = "jpg";
   OpenDialog1->FileName = "";
   OpenDialog1->Filter = "JPEG files (*.JPG)|*.JPG";
   OpenDialog1->Options << ofReadOnly;
   if (OpenDialog1->Execute())
   {
      strcpy(file_name,OpenDialog1->FileName.c_str());
      if (fg_jpeghead(file_name,file_header) < 0)
      {
         wsprintf(mb_text,"%s is not a JPEG file.",(LPSTR)file_name);
         Application->MessageBox(mb_text,"Error",MB_ICONSTOP|MB_OK);
         return;
      }
      Form1->Cursor = crHourGlass;
      colors = 0;
      buffsize = fg_jpegmem(file_header);
      buffer = new char[buffsize];
      fg_jpegbuf(buffer,buffsize);
      fg_jpegsize(file_header,&cxBuffer,&cyBuffer);
      switch_buffers();
      fg_move(0,0);
      fg_showjpeg(file_name,0);
      delete[] buffer;
      fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
      Form1->Cursor = crDefault;

      MakeBMP->Enabled = True;
      InfoBMP->Enabled = False;
      MakePCX->Enabled = True;
      InfoPCX->Enabled = False;
      PlayFLIC->Enabled = False;
      FrameFLIC->Enabled = False;
      ResetFLIC->Enabled = False;
      InfoFLIC->Enabled = False;
      InfoJPEG->Enabled = True;
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::InfoJPEGClick(TObject *Sender)
{
   MBText = "";
   MBText = MBText
            + file_name + "\n"
            + cxBuffer + "x" + cyBuffer + " pixels\n"
            + "24-bit RGB";
   ShowMessage(MBText);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::QuitClick(TObject *Sender)
{
   Application->Terminate();
}
//---------------------------------------------------------------------------
