!*****************************************************************************
!                                                                            *
!  KBDEMO.F90                                                                *
!                                                                            *
!  This program shows how to pan the contents of a virtual buffer through    *
!  a smaller window using the low-level keyboard handler.                    *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

include 'kbdemo.fi'

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGkbdemo'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW .or. CS_OWNDC
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = NULL
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Keyboard Handler Demo'C,&  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   WIDTH/2,                 &  ! initial x size
   HEIGHT/2,                &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

! The message loop processes entries placed in the message queue.
! When no message is ready, call check_for_panning() to check if
! we want to perform panning.

do while (.TRUE.)
   if (PeekMessage(msg,NULL,0,0,PM_REMOVE)) then
      if (msg.message == WM_QUIT) then
         WinMain = msg%wParam
         return
      else
         iret = TranslateMessage(msg)
         iret = DispatchMessage(msg)
      end if
   else
      call check_for_panning
   end if
end do

end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'kbdemo.fi'

type (T_PAINTSTRUCT) ps
integer*4 cxClient, cyClient
integer*4 hdc, hpal, hvb
integer*4 iret

integer*4 x, y, xlimit, ylimit
common    x, y, xlimit, ylimit
logical*1 CanGoLeft, CanGoRight, CanGoUp, CanGoDown
common    CanGoLeft, CanGoRight, CanGoUp, CanGoDown

save cxClient, cyClient, hdc, hpal, hvb

WindowProc = 0

select case (message)

   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      hvb = fg_vballoc(WIDTH,HEIGHT)
      iret = fg_vbopen(hvb)
      call fg_vbcolors

      iret = fg_showbmp('PORCH.BMP'C,0)
      x = 0
      y = 0
      CanGoRight = .TRUE.
      CanGoDown  = .TRUE.
      CanGoLeft  = .FALSE.
      CanGoUp    = .FALSE.
      return

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      if (cxClient < WIDTH) then
         xlimit = WIDTH - cxClient
         if (x > 0)      CanGoLeft = .TRUE.
         if (x < xlimit) CanGoRight = .TRUE.
      else
         xlimit = 0
         CanGoLeft  = .FALSE.
         CanGoRight = .FALSE.
      end if
      if (cyClient < HEIGHT) then
         ylimit = HEIGHT - cyClient
         if (y > 0)      CanGoUp = .TRUE.
         if (y < ylimit) CanGoDown = .TRUE.
      else
         ylimit = 0
         CanGoUp   = .FALSE.
         CanGoDown = .FALSE.
      end if
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return

end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************
!                                                                            *
!  The check_for_panning() routine checks if any of the four arrow keys are  *
!  pressed, and if so, pans in that direction if possible. It is called from *
!  the message loop in WinMain when no messages are waiting.                 *
!                                                                            *
!*****************************************************************************

subroutine check_for_panning
use fgwin

include 'kbdemo.fi'

integer*4 KB_ESCAPE, KB_LEFT, KB_RIGHT, KB_UP, KB_DOWN
parameter (KB_ESCAPE=1, KB_LEFT=75, KB_RIGHT=77, KB_UP=72, KB_DOWN=80)

integer*4 x, y, xlimit, ylimit
common    x, y, xlimit, ylimit
logical*1 CanGoLeft, CanGoRight, CanGoUp, CanGoDown
common    CanGoLeft, CanGoRight, CanGoUp, CanGoDown

if (fg_kbtest(KB_LEFT)==1 .AND. CanGoLeft) then
   if (x == xlimit) CanGoRight = .TRUE.
   x = x - 1
   call fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1)
   if (x == 0) CanGoLeft = .FALSE.

else if (fg_kbtest(KB_RIGHT)==1 .AND. CanGoRight) then
   if (x == 0) CanGoLeft = .TRUE.
   x = x + 1
   call fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1)
   if (x == xlimit) CanGoRight = .FALSE.

else if (fg_kbtest(KB_UP)==1 .AND. CanGoUp) then
   if (y == ylimit) CanGoDown = .TRUE.
   y = y - 1
   call fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1)
   if (y == 0) CanGoUp = .FALSE.

else if (fg_kbtest(KB_DOWN)==1 .AND. CanGoDown) then
   if (y == 0) CanGoUp = .TRUE.
   y = y + 1
   call fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1)
   if (y == ylimit) CanGoDown = .FALSE.

else if (fg_kbtest(KB_ESCAPE) == 1) then
   x = 0
   y = 0
   call fg_vbpaste(0,WIDTH-1,0,HEIGHT-1,0,HEIGHT-1)
   if (xlimit > 0) CanGoRight = .TRUE.
   if (ylimit > 0) CanGoDown = .TRUE.
   CanGoLeft = .FALSE.
   CanGoUp = .FALSE.
end if

return
end
